package com.fjhx.oa.service.adjust.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.oa.entity.adjust.dto.AdjustSalaryApplySelectDto;
import com.fjhx.oa.entity.adjust.po.AdjustSalaryApply;
import com.fjhx.oa.entity.adjust.po.AdjustSalaryDetail;
import com.fjhx.oa.entity.adjust.vo.AdjustSalaryApplyVo;
import com.fjhx.oa.entity.adjust.vo.AdjustSalaryDetailVo;
import com.fjhx.oa.mapper.adjust.AdjustSalaryApplyMapper;
import com.fjhx.oa.service.adjust.AdjustSalaryApplyService;
import com.fjhx.oa.service.adjust.AdjustSalaryDetailService;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 调薪申请 服务实现类
 * </p>
 *
 * @author
 * @since 2024-06-20
 */
@Service
public class AdjustSalaryApplyServiceImpl extends ServiceImpl<AdjustSalaryApplyMapper, AdjustSalaryApply> implements AdjustSalaryApplyService {

    @Resource
    private AdjustSalaryDetailService adjustSalaryDetailService;

    @Override
    public Page<AdjustSalaryApplyVo> getPage(AdjustSalaryApplySelectDto dto) {
        IWrapper<AdjustSalaryApply> wrapper = getWrapper();

        wrapper.keyword(dto.getKeyword(), new SqlField("su.nick_name"));


        //权限过滤 自己看自己，财务看全部
        List<String> userRoleKeys = UserUtil.getUserRoleKeys(SecurityUtils.getUserId());
        if (!userRoleKeys.contains("cfo")) {
            wrapper.eq("asa", AdjustSalaryApply::getUserId, SecurityUtils.getUserId());
        }

        wrapper.orderByDesc("asa", AdjustSalaryApply::getId);
        Page<AdjustSalaryApplyVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<AdjustSalaryApplyVo> records = page.getRecords();

        setInfo(records);
        return page;
    }

    @Override
    public AdjustSalaryApplyVo detail(Long id) {
        AdjustSalaryApply AdjustSalaryApply = this.getById(id);
        AdjustSalaryApplyVo result = BeanUtil.toBean(AdjustSalaryApply, AdjustSalaryApplyVo.class);

        setInfo(Collections.singletonList(result));
        return result;
    }

    private void setInfo(List<AdjustSalaryApplyVo> records) {
        if (ObjectUtil.isEmpty(records)) {
            return;
        }
        List<Long> ids = records.stream().map(AdjustSalaryApply::getId).collect(Collectors.toList());

        Map<Long, List<AdjustSalaryDetailVo>> collect = adjustSalaryDetailService.getList(q -> q.in(AdjustSalaryDetail::getAdjustSalaryId, ids))
                .stream().collect(Collectors.groupingBy(AdjustSalaryDetail::getAdjustSalaryId));
        for (AdjustSalaryApplyVo record : records) {
            List<AdjustSalaryDetailVo> adjustSalaryDetailVos = collect.get(record.getId());
            record.setUserSalaryDetailList(adjustSalaryDetailVos);
        }
        UserUtil.assignmentNickName(records, AdjustSalaryApply::getUserId, AdjustSalaryApplyVo::setUserName);
    }

}
